/**********************************************************************
Copyright (c) 2009-2010 Alexander Kerner. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 ***********************************************************************/

package net.sf.jcmdlineparser;

/**
 * <p>
 * An {@code AbstractCounter} can be used to monitor program behavior or to keep
 * track of some progress, e.g. how many loop cycles have already been
 * performed.
 * </p>
 * <p>
 * This class is fully threadsave.
 * </p>
 * 
 * <p>
 * Example:
 * 
 * <pre>
 * AbstractCounter cnt = new AbstractCounter() {
 * 	&#064;Override
 * 	public void perform() {
 * 		System.out.println(&quot;performed!&quot;);
 * 	}
 * };
 * cnt.setInterval(2);
 * for (int i = 0; i &lt; 10; i++) {
 * 	cnt.count();
 * }
 * </pre>
 * </p>
 * 
 * @author <a href="mailto:alex.kerner.24@googlemail.com">Alexander Kerner</a>
 * @version 2010-10-01
 * 
 * 
 */
public abstract class AbstractCounter {

	// Field //

	private volatile int interval = 0;
	private volatile int intervalHelper = 0;
	private volatile int cnt = 0;
	private final int initCnt;

	// Constructor //

	/**
	 * Create a new {@code AbstractCounter} that has initially performed zero
	 * counts.
	 */
	public AbstractCounter() {
		// all zero
		initCnt = 0;
	}

	/**
	 * <p>
	 * Construct a new {@code AbstractCounter} with has initially performed
	 * {@code count} counts. {@link AbstractCounter#getCount()} will return
	 * {@code count}.
	 * </p>
	 * 
	 * @param count
	 *            Initial counting for this {@code AbstractCounter}.
	 */
	public AbstractCounter(int count) {
		synchronized (AbstractCounter.class) {
			this.cnt = count;
			this.intervalHelper = count;
			this.initCnt = count;
		}
	}

	// Private //

	private void checkPerform() {
		if (intervalHelper >= interval) {
			perform();
			intervalHelper = 0;
		}
	}

	// Protected //

	// Public //

	/**
	 * <p>
	 * Increase counter by 1.
	 * </p>
	 */
	public synchronized void count() {
		if(cnt == Integer.MAX_VALUE)
			throw new ArithmeticException("Integer overflow");
		cnt++;
		intervalHelper++;
		checkPerform();
	}

	/**
	 * <p>
	 * Resets this {@code AbstractCounter} and calls
	 * {@link AbstractCounter#perform()} if there has been any counts after last
	 * call of {@link AbstractCounter#perform()}.
	 * </p>
	 */
	public synchronized void finish() {
		if (interval != 0)
			perform();
		intervalHelper = initCnt;
		cnt = initCnt;
		interval = 0;
	}

	/**
	 * <p>
	 * Sets {@code interval} for this {@code AbstractCounter}.
	 * </p>
	 * 
	 * @param interval
	 *            number of counts {@link AbstractCounter#count()} has to be
	 *            called before {@link AbstractCounter#perform()} is called.
	 * @throws NumberFormatException
	 *             if {@code interval} < 1
	 */
	public synchronized void setInterval(int interval) {
		if (interval < 0)
			throw new NumberFormatException("interval must be >= 0");
		this.interval = interval;
	}

	/**
	 * @return current interval for this {@code AbstractCounter}.
	 */
	public int getInterval() {
		return interval;
	}

	/**
	 * <p>
	 * Returns the number of {@link AbstractCounter#count()} has been called.
	 * </p>
	 * 
	 * @return current counts
	 */
	public int getCount() {
		return cnt;
	}

	/**
	 * <p>
	 * Returns the init value of {@link AbstractCounter#cnt}.
	 * </p>
	 * 
	 * @return init count
	 */
	public int getInitCount() {
		return initCnt;
	}

	/**
	 * 
	 * <p>
	 * Set {@code count}.
	 * </p>
	 * 
	 * 
	 * @param count
	 *            {@code count} to set
	 */
	public synchronized void setCount(int count) {
		this.cnt = count;
		this.intervalHelper = count;
	}

	// Override //

	@Override
	public String toString() {
		return Integer.toString(getCount());
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + cnt;
		result = prime * result + interval;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof AbstractCounter))
			return false;
		AbstractCounter other = (AbstractCounter) obj;
		if (cnt != other.cnt)
			return false;
		if (interval != other.interval)
			return false;
		return true;
	}

	// Abstract //

	/**
	 * <p>
	 * Performs action.
	 * </p>
	 */
	public abstract void perform();

}
