/**********************************************************************
Copyright (c) 2009-2010 Alexander Kerner. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 ***********************************************************************/

package net.sf.jcmdlineparser.helpprinter;

import java.io.OutputStream;
import java.util.Set;

import net.sf.jcmdlineparser.options.AbstractOption;
import net.sf.jcmdlineparser.parser.CmdLineParser;

/**
 * <p>
 * A {@code HelpPrinter} is used to build and print the help-string.
 * </p>
 * 
 * <p>
 * Example:
 * <p>
 * When implementing {@link net.sf.jcmdlineparser.parser.CmdLineParser
 * CmdLineParser#printHelp(String)} and
 * {@link net.sf.jcmdlineparser.parser.CmdLineParser
 * CmdLineParser#printHelp(String, OutputStream)}, use {@code HelpPrinter} for
 * delegation:
 * </p>
 * 
 * <pre>
 * public void printHelp(String usageString) {
 * 	new SimpleHelpPrinter().printHelp(options, usageString);
 * }
 * 
 * public void printHelp(String usageString, OutputStream out) {
 * 	new SimpleHelpPrinter().printHelp(options, usageString, out);
 * }
 * </pre>
 * 
 * </p>
 * 
 * @author <a href="mailto:alex.kerner.24@googlemail.com">Alexander Kerner</a>
 * @see CmdLineParser
 * @version 2010-08-10
 * 
 */
public interface HelpPrinter {

	// lastVisit 2010-08-10

	/**
	 * <p>
	 * Prints a formatted help text to STDOUT.
	 * </p>
	 * 
	 * @param options
	 *            command line options that are registered and which will be
	 *            considered in help text
	 * @param usageString
	 *            a prefix string, that gives short summary on general pogram
	 *            usage
	 */
	void printHelp(Set<AbstractOption<?>> options, String usageString);

	/**
	 * <p>
	 * Prints a formatted help text to {@link java.io.OutputStream OutputStream} <code>out<code>.
	 * </p>
	 * 
	 * @param options
	 *            command line options that are registered and which will be
	 *            considered in help text
	 * @param usageString
	 *            a prefix string, that gives short summary on general program
	 *            usage
	 * @param out
	 *            {@link java.io.OutputStream OutputStream} help text is printed to.
	 */
	void printHelp(Set<AbstractOption<?>> options, String usageString,
			OutputStream out);

}
