/**********************************************************************
Copyright (c) 2009-2010 Alexander Kerner. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 ***********************************************************************/

package net.sf.jcmdlineparser.options;

import java.util.LinkedHashSet;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import net.sf.jcmdlineparser.ParserException;

/**
 * <p>
 * An {@code AbstractOption} represents a command line option.
 * </p>
 * 
 * @author <a href="mailto:alex.kerner.24@googlemail.com">Alexander Kerner</a>
 * @version 2010-08-10
 * 
 */
public abstract class AbstractOption<T> {

	private final Logger log = LoggerFactory.getLogger(AbstractOption.class);

	/**
	 * <p>
	 * String based identifier for this option.
	 * </p>
	 */
	protected final String identifierLong;

	/**
	 * <p>
	 * Character based identifier for this option.
	 * </p>
	 */
	protected final char identifierShort;

	/**
	 * <p>
	 * Description for this option.
	 * </p>
	 */
	protected final String description;

	/**
	 * *
	 * <p>
	 * Return type for option value(s).
	 * </p>
	 */
	protected final Class<T> returnType;

	/**
	 * <p>
	 * This-option-is-required-flag.
	 * </p>
	 */
	protected final boolean required;

	/**
	 * <p>
	 * This-option-was-set-flag.
	 * </p>
	 */
	protected volatile boolean set = false;

	/**
	 * <p>
	 * Other options that clash (cannot coexist) with this option.
	 * </p>
	 */
	protected final Set<AbstractOption<?>> clashOptions = new LinkedHashSet<AbstractOption<?>>();

	/**
	 * <p>
	 * Other options that are required by this option.
	 * </p>
	 */
	protected final Set<AbstractOption<?>> requiredOptions = new LinkedHashSet<AbstractOption<?>>();

	/**
	 * <p>
	 * TODO
	 * </p>
	 */
	protected AbstractOption(Class<T> returnType, char identifierShort,
			String identifierLong, String description, boolean required) {
		synchronized (AbstractOption.class) {
			if (returnType == null)
				throw new NullPointerException();
			if (!Character.isLetterOrDigit(identifierShort))
				throw new IllegalArgumentException(
						"identifier must be a letter or digit");
			this.returnType = returnType;
			this.identifierShort = identifierShort;
			if (identifierLong != null)
				this.identifierLong = identifierLong;
			else
				this.identifierLong = Character.toString(identifierShort);
			this.description = description;
			this.required = required;
		}
	}

	/**
	 * <p>
	 * TODO
	 * </p>
	 */
	protected AbstractOption(Class<T> returnType, char identifierShort,
			String identifierLong, String description, boolean required,
			Set<AbstractOption<?>> clashOptions) {
		if (returnType == null)
			throw new NullPointerException();
		if (!Character.isLetterOrDigit(identifierShort))
			throw new IllegalArgumentException(
					"identifier must be a letter or digit");
		this.returnType = returnType;
		this.identifierShort = identifierShort;
		if (identifierLong != null)
			this.identifierLong = identifierLong;
		else
			this.identifierLong = Character.toString(identifierShort);
		this.description = description;
		this.required = required;
		if (clashOptions != null && !clashOptions.isEmpty())
			this.clashOptions.addAll(clashOptions);
	}

	// Override //

	public synchronized int hashCode() {
		int r = 17;
		r = 31 * r + identifierLong.hashCode();
		r = 31 * r + identifierShort;
		return r;
	}

	public synchronized String toString() {
		return identifierLong + "(" + identifierShort + ")";
	}

	public synchronized boolean equals(Object o) {
		if (o instanceof AbstractOption<?>) {
			AbstractOption<?> cp = (AbstractOption<?>) o;
			if (cp.identifierLong.equals(this.identifierLong)
					|| cp.identifierShort == this.identifierShort)
				return true;
		}
		return false;
	}

	// Public //

	/**
	 * 
	 * <p>
	 * Defines another {@code AbstractOption}, that will exclude this
	 * {@code AbstractOption} to be set and vice versa. Calling this method will
	 * also call {@link AbstractOption#addClashOption(AbstractOption)} on
	 * {@code clashOption}.
	 * <p>
	 * 
	 * @param clashOption
	 *            {@code AbstractOption} that clashes with this option
	 */
	public synchronized void addClashOption(AbstractOption<?> clashOption) {
		if (clashOptions.add(clashOption)) {
			// all good
			clashOption.addClashOption(this);
		} else {
			log.debug("clashoption \"" + clashOption
					+ "\" was already registered");
		}
	}

	/**
	 * <p>
	 * Gets all clashing {@code AbstractOption}s for this {@code AbstractOption}
	 * , that means all {@code AbstractOption} that cannot coexist with this
	 * {@code AbstractOption}.
	 * </p>
	 * 
	 * @return a {@code Set} of {@code AbstractOption}'s that clash with this
	 *         {@code AbstractOption}
	 */
	public synchronized Set<AbstractOption<?>> getClashOptions() {
		return new LinkedHashSet<AbstractOption<?>>(clashOptions);
	}

	/**
	 * <p>
	 * Gets all required {@code AbstractOption} for this {@code AbstractOption},
	 * that means all {@code AbstractOption}s that must be set if this
	 * {@code AbstractOption} is set.
	 * </p>
	 * 
	 * @return a {@code Set} of {@code AbstractOption}'s that are required by
	 *         this option
	 */
	public synchronized Set<AbstractOption<?>> getRequiredOptions() {
		return new LinkedHashSet<AbstractOption<?>>(requiredOptions);
	}

	/**
	 * 
	 * <p>
	 * Defines another {@code AbstractOption} that is required by this
	 * {@code AbstractOption}.
	 * <p>
	 * 
	 * @param requiredOption
	 *            {@code AbstractOption} that is required by this
	 *            {@code AbstractOption}
	 */
	public synchronized void addRequiredOption(AbstractOption<?> requiredOption) {
		if (requiredOptions.add(requiredOption)) {
			// all good
		} else {
			log.debug("required option \"" + requiredOption
					+ "\" was already registered");
		}
	}

	/**
	 * 
	 * @return the long identifier ({@code String} representation) for this
	 *         {@code AbstractOption}
	 */
	public String getKeyLong() {
		return identifierLong;
	}

	/**
	 * 
	 * @return the short identifier ({@code character} representation) for this
	 *         {@code AbstractOption}
	 */
	public char getKeyShort() {
		return identifierShort;
	}

	/**
	 * 
	 * @return the description for this {@code AbstractOption}
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * 
	 * @return true, if this {@code AbstractOption} was set on the command line;
	 *         false otherwise
	 */
	public boolean isSet() {
		return set;
	}

	/**
	 * <p>
	 * Mark this {@code AbstractOption} as set on the command line
	 * </p>
	 * 
	 * @param set
	 *            true if this {@code AbstractOption} was set on the command
	 *            line, false otherwise
	 */
	protected void setSet(boolean set) {
		this.set = set;
	}

	/**
	 * 
	 * @return true, if this {@code AbstractOption} has been declared to be
	 *         explicitly required
	 */
	public boolean isRequired() {
		return required;
	}

	/**
	 * 
	 * TODO
	 */
	public Class<T> getReturnType() {
		return returnType;
	}

	// Abstract //

	/**
	 * <p>
	 * Parses value(s) for this {@code AbstractOption}.
	 * </p>
	 * 
	 * @param string
	 *            String that is parsed
	 * @throws NullPointerException
	 *             if given String is null
	 * @throws ParserException
	 *             if no valid values can be parsed from this string
	 */
	public abstract void parse(String string) throws ParserException;

}
