/**********************************************************************
Copyright (c) 2009-2010 Alexander Kerner. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 *********************************************************************/

package net.sf.jcmdlineparser.strategy;

import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.sf.jcmdlineparser.ParserException;
import net.sf.jcmdlineparser.options.AbstractOption;
import net.sf.jcmdlineparser.options.NoSuchOptionException;
import net.sf.jcmdlineparser.options.NoValueGivenException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * <p>
 * Most simple implementation for {@link Strategy}.
 * </p>
 * <p>
 * Accepted format is
 * 
 * <blockquote> k=v key=value ... </blockquote>
 * 
 * </p>
 * 
 * @author <a href="mailto:alex.kerner.24@googlemail.com">Alexander Kerner</a>
 * @version 2010-08-10
 * @see AbstractOption
 * @see Strategy
 * 
 */
public class SimpleStrategy implements Strategy {

	// lastVisit 2010-08-11

	private final static Logger logger = LoggerFactory.getLogger(SimpleStrategy.class);

	private final static Pattern pattern = Pattern.compile("(.+)=(.*)");

	// Implement //

	public String[] parse(String[] string, Set<AbstractOption<?>> options)
			throws ParserException {
		final StringBuilder sb = new StringBuilder();
		for (String s : string) {
			// + " " important !!
			sb.append(s + " ");
		}
		return parse(sb.toString(), options).split(" ");
	}

	public String parse(String string, Set<AbstractOption<?>> options)
			throws ParserException {
		if (string == null || options == null || string.length() < 1)
			throw new NullPointerException("empty string or no options given");
		logger.debug("parsing string: \"" + string
				+ "\", options to process: \"" + options + "\"");

		// elements, that could not be recognized.
		final StringBuilder unknown = new StringBuilder();

		final String[] s = string.split(" ");
		for (String s2 : s) {
			Matcher m = pattern.matcher(s2);
			// found pattern, at least key available
			if (m.matches()) {
				final String key = m.group(1);
				final String value = m.group(2);
				logger.debug("Found matching : " + s2 + ", key: " + key
						+ ", value: " + value);
				if(value == null || value.length() < 1)
					throw new NoValueGivenException();
				try {
					StrategyUtils.handleKeyValuePair(key, value, options);
				} catch (NoSuchOptionException e) {
					logger.error(e.getLocalizedMessage(), e);
					logger.debug(e + ", adding to unknown");
					unknown.append(s2 + " ");
				}
				// pattern does not match, add to unknown
			} else {
				logger.debug("\"" + s2 + "\" does not match expression");
				unknown.append(s2 + " ");
			}
		}

		logger.debug("parsing done, unknwon: \"" + unknown.toString().trim()
				+ "\"");
		return unknown.toString().trim();
	}
}
